#include "Bullet.h"
#include "sprite.h"
#include "Utils.h"
#include "Player.h"
#include "ObjectManager.h"

void BulletUpdate(ObjectStruct *);
void BulletDestroy(ObjectStruct *);
void BulletDraw(ObjectStruct *);
void BulletCollide(ObjectStruct *, ObjectStruct *);

ObjectStruct *BulletCreate(fix16 x, fix16 y)
{
    ObjectStruct *object = ObjectManager_CreateObject(TYPE_ENEMY_PROJECTILE,
                                                      BulletUpdate,
                                                      BulletDestroy,
                                                      BulletDraw,
                                                      BulletCollide);

    if (!object)
    {
        KLog("Bullet could not be created! 1");
        return NULL;
    }

    object->x = x;
    object->y = y;
    object->speedX = 0;
    object->speedY = 0;
    object->accelX = FIX16(0.1);
    object->accelY = FIX16(0.1);
    object->animationFrame = 0;
    object->animationSubFrameCounter = 0;
    object->healthPoints = 1;
    object->damage = 1;
    object->spriteDef = &enemybullet;

    if (!AddObjectPosition(object))
    {
        KLog("BulletCreate could not be created! 1");
        return NULL;
    }

    return object;
}

void BulletUpdate(ObjectStruct *object)
{
    if (object->objectState == OBJECTSTATE_INACTIVE)
    {
        return;
    }

    object->animationSubFrameCounter++;
    if (object->animationSubFrameCounter > 10)
    {
        object->animationFrame++;
        if (object->animationFrame > 1)
        {
            object->animationFrame = 0;
        }
        object->animationSubFrameCounter = 0;
        SPR_setFrame(object->sprite, object->animationFrame);
    }

    // if (object->flags & BULLET_PATH_FOLLOW)
    // {
    //     if ((playerObject->x < object->x && object->maxSpeedX > 0) || (playerObject->x > object->x && object->maxSpeedX < 0))
    //     {
    //         object->maxSpeedX = -object->maxSpeedX;
    //     }
    // }

    if (object->speedX < object->maxSpeedX)
        object->speedX += object->accelX;
    if (object->speedX > object->maxSpeedX)
        object->speedX -= object->accelX;

    if (object->speedY < object->maxSpeedY)
        object->speedY += object->accelY;
    if (object->speedY > object->maxSpeedY)
        object->speedY -= object->accelY;

    object->x += object->speedX;
    object->y += object->speedY;

    SetObjectPosition(object);
}

void BulletDestroy(ObjectStruct *object)
{
}

void BulletDraw(ObjectStruct *object)
{
}

void BulletCollide(ObjectStruct *object, ObjectStruct *collidedObject)
{
    object->objectState = OBJECTSTATE_INACTIVE;
}
