#include "Enemy2.h"
#include "sprite.h"
#include "Utils.h"
#include "BarrageManager.h"
#include "Constants.h"
#include "ObjectManager.h"
#include "Explosion.h"

void Enemy2Update(ObjectStruct *);
void Enemy2Destroy(ObjectStruct *);
void Enemy2Draw(ObjectStruct *);
void Enemy2Collide(ObjectStruct *, ObjectStruct *);

ObjectStruct *Enemy2Create(fix16 x, fix16 y)
{
    ObjectStruct *object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                      Enemy2Update,
                                                      Enemy2Destroy,
                                                      Enemy2Draw,
                                                      Enemy2Collide);

    if (!object)
    {
        KLog("Enemy2 could not be created! 1");
        return NULL;
    }

    object->x = x;
    object->y = y;
    object->speedX = 0;
    object->speedY = 0;
    object->accelX = FIX16(0.1);
    object->accelY = FIX16(0.2);
    object->animationFrame = 0;
    object->animationSubFrameCounter = 0;
    object->healthPoints = 20;
    object->spriteDef = &enemymedium;
    object->palette = PAL0;
    object->invulnerableTimer = 0;
    object->isAlive = 1;
    object->lifetime = 0;
    object->shotTimer = random() % 3 + 2;
    object->shotsFired = 0;
    object->speed = FIX16(0.3);
    object->score = object->healthPoints;

    if (!AddObjectPosition(object))
    {
        KLog("Enemy2 could not be created! 1");
        return NULL;
    }

    return object;
}

void Enemy2Update(ObjectStruct *object)
{
    if (object->objectState == OBJECTSTATE_INACTIVE)
    {
        return;
    }

    CommonEnemyUpdate(object);

    object->shotTimer--;
    if (object->shotTimer <= 0)
    {
        u8 rand = random() % 100;
        if (rand > 80)
        {
            BarrageManager_CreateBarrage(SHOOT6x, object);
            object->shotTimer = (random() % 50) + 160;
        }
        else
        {
            BarrageManager_CreateBarrage(SHOOT3x, object);
            object->shotTimer = (random() % 50) + 80;
        }
    }
}

void Enemy2Destroy(ObjectStruct *object)
{
    SPR_releaseSprite(object->sprite);
    enemyNumber--;
}

void Enemy2Draw(ObjectStruct *object)
{
    SPR_setFrame(object->sprite, object->animationFrame);
}

void Enemy2Collide(ObjectStruct *object, ObjectStruct *collidedObject)
{
    if (object->invulnerableTimer > 0)
    {
        return;
    }

    if (!object->isAlive)
    {
        return;
    }

    if (collidedObject->objectType == TYPE_PLAYER)
    {
        return;
    }

    object->healthPoints -= collidedObject->damage;

    object->invulnerableTimer = 5;

    if (object->healthPoints <= 0)
    {
        s16 halfHeight = object->sprite->frame->h >> 1;
        s16 halfWidth = object->sprite->frame->w >> 1;
        fix16 x = object->x + intToFix16(halfWidth);
        fix16 y = object->y + intToFix16(halfHeight);
        ExplosionCreate(x, y);
        ExplosionCreate(x + intToFix16(8), y);
        object->isAlive = 0;
        object->objectState = OBJECTSTATE_INACTIVE;
    }
    else
    {
        SPR_setPalette(object->sprite, PAL2);
    }
}
