#include "Enemy3.h"
#include "sprite.h"
#include "Utils.h"
#include "BarrageManager.h"
#include "Constants.h"
#include "ObjectManager.h"
#include "Explosion.h"

void Enemy3Update(ObjectStruct *);
void Enemy3Destroy(ObjectStruct *);
void Enemy3Draw(ObjectStruct *);
void Enemy3Collide(ObjectStruct *, ObjectStruct *);

ObjectStruct *Enemy3Create(fix16 x, fix16 y)
{
    ObjectStruct *object = ObjectManager_CreateObject(TYPE_ENEMY,
                                                      Enemy3Update,
                                                      Enemy3Destroy,
                                                      Enemy3Draw,
                                                      Enemy3Collide);

    if (!object)
    {
        KLog("Enemy3 could not be created! 1");
        return NULL;
    }

    object->x = x;
    object->y = y;
    object->speedX = 0;
    object->speedY = 0;
    object->accelX = FIX16(0.05);
    object->accelY = FIX16(0.1);
    object->animationFrame = 0;
    object->animationSubFrameCounter = 0;
    object->healthPoints = 30;
    object->spriteDef = &enemybig;
    object->palette = PAL0;
    object->invulnerableTimer = 0;
    object->isAlive = 1;
    object->lifetime = 0;
    object->shotTimer = random() % 3 + 2;
    object->speed = FIX16(0.1);
    object->score = object->healthPoints;

    if (!AddObjectPosition(object))
    {
        KLog("Enemy3 could not be created! 1");
        return NULL;
    }

    return object;
}

void Enemy3Update(ObjectStruct *object)
{
    if (object->objectState == OBJECTSTATE_INACTIVE)
    {
        return;
    }

    CommonEnemyUpdate(object);

    object->shotTimer--;
    if (object->shotTimer <= 0)
    {
        u8 rand = random() % 100;
        // if (rand > 80)
        // {
        //     object->shotTimer = (random() % 50) + 200;
        //     BarrageManager_CreateBarrage(SHOOT9x, object);
        // }
        // else
        // {
            object->shotTimer = (random() % 50) + 100;
            BarrageManager_CreateBarrage(SHOOT6x, object);
        //}
    }
}

void Enemy3Destroy(ObjectStruct *object)
{
    SPR_releaseSprite(object->sprite);
    enemyNumber--;
}

void Enemy3Draw(ObjectStruct *object)
{
    SPR_setFrame(object->sprite, object->animationFrame);
}

void Enemy3Collide(ObjectStruct *object, ObjectStruct *collidedObject)
{
    if (object->invulnerableTimer > 0)
    {
        return;
    }

    if (!object->isAlive)
    {
        return;
    }

    if (collidedObject->objectType == TYPE_PLAYER)
    {
        return;
    }

    object->healthPoints -= collidedObject->damage;

    object->invulnerableTimer = 5;

    if (object->healthPoints <= 0)
    {
        s16 halfHeight = object->sprite->frame->h >> 1;
        s16 halfWidth = object->sprite->frame->w >> 1;
        fix16 x = object->x + intToFix16(halfWidth);
        fix16 y = object->y + intToFix16(halfHeight);
        ExplosionCreate(x, y);
        ExplosionCreate(x + intToFix16(8), y);
        ExplosionCreate(x + intToFix16(8), y + intToFix16(8));
        ExplosionCreate(x, y + intToFix16(8));
        object->isAlive = 0;
        object->objectState = OBJECTSTATE_INACTIVE;
    }
    else
    {
        SPR_setPalette(object->sprite, PAL2);
    }
}
