#include "Explosion.h"
#include "sprite.h"

#include "Utils.h"

void ExplosionUpdate(ObjectStruct *explosionObject);
void ExplosionDestroy(ObjectStruct *explosionObject);
void ExplosionDraw(ObjectStruct *explosionObject);

ObjectStruct *ExplosionCreate(fix16 x, fix16 y)
{
    ObjectStruct *object = ObjectManager_CreateObject(TYPE_EFFECT,
                                                      ExplosionUpdate,
                                                      ExplosionDestroy,
                                                      ExplosionDraw,
                                                      CollisionFunctionStub);

    if (!object)
    {
        KLog("Explosion could not be created! 1");
        return NULL;
    }

    object->x = x;
    object->y = y;
    object->animationFrame = 0;
    object->animationSubFrameCounter = 0;
    object->healthPoints = 1;
    object->spriteDef = &explosion;
    if (!AddObjectPosition(object))
    {
        KLog("Explosion could not be created! 1");
        return NULL;
    }
    return object;
}

void ExplosionUpdate(ObjectStruct *object)
{
    if (object->objectState == OBJECTSTATE_INACTIVE)
    {
        return;
    }

    object->animationSubFrameCounter++;

    if (object->animationSubFrameCounter > 4)
    {
        object->animationFrame++;
        if (object->animationFrame > 4)
        {
            object->objectState = OBJECTSTATE_INACTIVE;
            return;
        }
        SPR_setFrame(object->sprite, object->animationFrame);
        object->animationSubFrameCounter = 0;
    }

    SetObjectPosition(object);
}

void ExplosionDestroy(ObjectStruct *object)
{
    SPR_releaseSprite(object->sprite);
}

void ExplosionDraw(ObjectStruct *explosionObject)
{
}
