#include "LevelManager.h"
#include "Palette.h"
#include "Player.h"
#include <genesis.h>
#include "ObjectSpawner.h"
#include "JoypadManager.h"
#include "gfx.h"
#include "sprite.h"
#include "Utils.h"

enum LEVEL_STATE
{
    TITLE_LOADING,
    TITLE_UPDATING,
    LOADING,
    UPDATING
};

u16 baseTile;

u16 halfMapWidth;
u16 halfMapHeight;

Map *currentMap;

enum LEVEL_STATE state;

Sprite *HUD[3];
Sprite *HUD_WEAPON;
Sprite *HUD_HP[10];
Sprite *HUD_MP[10];
Sprite *HUD_CHECKER[16];
Sprite *laserSprite;

s16 x[4];

u16 frames = 0;
u16 score;
char scoreStr[10];

Sprite *titleSprites[6];

void StartGame()
{
    score = 0;
    frames = 0;
    DestroyObjects();
    if (state == TITLE_UPDATING)
    {
        VDP_clearPlan(PLAN_A, TRUE);
        VDP_clearPlan(PLAN_B, TRUE);
    }
    VDP_setHorizontalScroll(PLAN_B, 0);
    VDP_setHorizontalScroll(PLAN_A, 0);
    SPR_clear();
    SPR_end();
    SPR_init();
    VDP_setPlanSize(64, 32);
    ObjectManager_Init();
    state = LOADING;
}

void LevelManagerInit()
{
    frames = 0;
    state = TITLE_LOADING;
    //StartGame();
}

void UpdateHP(u8 hp)
{
    SPR_setVisibility(HUD_HP[10 - hp], HIDDEN);
}

void UpdateMP(u8 mp)
{
    for (u8 i = 0; i < mp; i++)
    {
        SPR_setVisibility(HUD_MP[i], VISIBLE);
    }
}

void AddScore(u8 score_)
{
    score += score_ << 2;
    sprintf(scoreStr, "%d", score);
    VDP_drawText(scoreStr, 2, 1);
}

void LevelManagerUpdate()
{
    switch (state)
    {
    case TITLE_LOADING:
        SetPaletteAllBlack();

        halfMapWidth = (currentMap->w) >> 1;
        halfMapHeight = (currentMap->h) >> 1;

        u16 ind = TILE_USERINDEX;
        VDP_drawImageEx(PLAN_B, &title_background, TILE_ATTR_FULL(PAL0, FALSE, FALSE, FALSE, ind), 0, 0, FALSE, TRUE);
        ind += title_background.tileset->numTile;
        VDP_drawImageEx(PLAN_A, &title_background2, TILE_ATTR_FULL(PAL0, FALSE, FALSE, FALSE, ind), 0, 0, FALSE, TRUE);

        titleSprites[0] = SPR_addSprite(&title_pilot1x1, SCREEN_WIDTH, SCREEN_HEIGHT - 132, TILE_ATTR(PAL3, FALSE, FALSE, FALSE));
        titleSprites[1] = SPR_addSprite(&title_pilot1x2, SCREEN_WIDTH, SCREEN_HEIGHT - 8, TILE_ATTR(PAL3, FALSE, FALSE, FALSE));
        titleSprites[2] = SPR_addSprite(&title1x1, 0, 0, TILE_ATTR(PAL2, FALSE, FALSE, FALSE));
        titleSprites[3] = SPR_addSprite(&title2x1, 128, 0, TILE_ATTR(PAL2, FALSE, FALSE, FALSE));

        titleSprites[4] = SPR_addSprite(&planet1, 220, SCREEN_HEIGHT - 88, TILE_ATTR(PAL1, FALSE, FALSE, FALSE));

        SetPalette(PAL0, title_background.palette->data);
        SetPalette(PAL1, planet1.palette->data);
        SetPalette(PAL2, title1x1.palette->data);
        SetPalette(PAL3, title_pilot1x1.palette->data);

        x[0] = 220;
        x[1] = 40;

        //x[2] = SCREEN_WIDTH;
        x[2] =SCREEN_WIDTH - 140;
        x[3] = 0;

        VDP_setTextPalette(PAL1);
        VDP_setTextPlan(PLAN_B);
        VDP_drawText("Start Game", 7, 15);

        VDP_setTextPalette(PAL0);
        VDP_drawText("Options", 7, 17);
        VDP_drawText("Credits", 7, 19);

        FadeIn();
        state = TITLE_UPDATING;
        break;
    case TITLE_UPDATING:
        x[0] = frames >> 4;
        x[1] = -frames >> 5;

        // if (x[2] > SCREEN_WIDTH - 140)
        //     x[2] -= 1;
        // else
        //     x[2] = SCREEN_WIDTH - 140;

        if (x[3] < 44)
            x[3] = frames >> 2;
        else
            x[3] = 44;

        SPR_setPosition(titleSprites[0], x[2], SCREEN_HEIGHT - 132);
        SPR_setPosition(titleSprites[1], x[2], SCREEN_HEIGHT - 8);
        SPR_setPosition(titleSprites[2], x[3], 0);
        SPR_setPosition(titleSprites[3], x[3] + 128, 0);

        SPR_setPosition(titleSprites[4], 220 + x[0], SCREEN_HEIGHT - 88 - 30);

        if (joystickState.aPressed)
        {
            StartGame();
            return;
        }

        VDP_setHorizontalScroll(PLAN_A, -frames >> 5);
        break;
    case LOADING:
        ObjectSpawner_Init();
        SetPaletteAllBlack();

        halfMapWidth = (currentMap->w) >> 1;
        halfMapHeight = (currentMap->h) >> 1;

        ind = TILE_USERINDEX;
        VDP_drawImageEx(PLAN_B, &level1b, TILE_ATTR_FULL(PAL1, FALSE, FALSE, FALSE, TILE_USERINDEX), 8, 0, FALSE, TRUE);
        ind += level1b.tileset->numTile;
        VDP_drawImageEx(PLAN_A, &planatrim, TILE_ATTR_FULL(PAL2, FALSE, FALSE, FALSE, ind), 0, 2, FALSE, TRUE);

        laserSprite = SPR_addSprite(&laser, 28, 110 + 12, TILE_ATTR(PAL3, TRUE, FALSE, FALSE));

        for (u8 i = 0; i < 10; i++)
        {
            HUD_HP[i] = SPR_addSprite(&hp, 18, i * 9 + 20, TILE_ATTR(PAL3, TRUE, FALSE, FALSE));
            HUD_MP[i] = SPR_addSprite(&mp, 39, i * 9 + 20, TILE_ATTR(PAL3, TRUE, FALSE, FALSE));
            SPR_setVisibility(HUD_MP[i], HIDDEN);
        }

        PlayerInit();

        SetPalette(PAL0, enemysmall.palette->data);
        SetPalette(PAL1, level1b.palette->data);
        SetPalette(PAL2, planatrim.palette->data);
        SetPalette(PAL3, ship.palette->data);

        VDP_setTextPalette(PAL2);
        VDP_setTextPlan(PLAN_A);
        VDP_drawText("Score", 2, 0);
        VDP_drawText("0", 2, 1);

        FadeIn();
        state = UPDATING;
        break;
    case UPDATING:
        VDP_setVerticalScroll(PLAN_B, -playerObject->lifetime);
        ObjectManager_UpdateObjects();
        ObjectSpawner_Update();
        ObjectManager_DrawObjects();
        break;
    }
    frames++;
}