#ifndef OBJECTSTRUCT_INCLUDE_H
#define OBJECTSTRUCT_INCLUDE_H

#include <genesis.h>
#include "Constants.h"

#define OBJECTSTATE_ACTIVE 0
#define OBJECTSTATE_INACTIVE 1

#define AISTATE_PURSUING 0
#define AISTATE_FLEEING 1

// object types.
#define TYPE_PLAYER_PROJECTILE 0
#define TYPE_EFFECT 1
#define TYPE_ENEMY 2
#define TYPE_ENEMY_PROJECTILE 3
#define TYPE_PICKUP 4
#define TYPE_PLATFORM 5
#define MAX_OBJECT_TYPES 6
// specially treated object types.
#define TYPE_NONE 6
#define TYPE_PLAYER 7

#define MAX_SPRITES 3

typedef struct
{
    fix16 x;
    fix16 y;
} Vect2D_f32;

typedef struct object_struct
{
    s16 objectStreamerId;

    struct object_struct *nextVisibleObjectOfSameType;

    void (*updateFunction)(struct object_struct *objectStruct);
    void (*destroyFunction)(struct object_struct *objectStruct);
    void (*drawFunction)(struct object_struct *objectStruct);
    void (*collisionFunction)(struct object_struct *objectStruct, struct object_struct *collidedObject);

    fix16 x;
    fix16 y;

    fix16 maxSpeedX;
    fix16 maxSpeedY;
    fix16 speedX;
    fix16 speedY;
    fix16 accelX;
    fix16 accelY;

    fix16 speed;

    u8 objectType;
    u16 lifetime; // one tick every frame. 60 ticks every second. 3600 ticks every minute. Max 18.2 minutes. only for temporary objects.

    u16 objectState;

    s16 rectLeft;
    s16 rectRight;
    s16 rectTop;
    s16 rectBottom;

    u8 healthPoints;
    u8 damage;

    u8 isAlive;

    u16 animationFrame;
    u16 animationSubFrameCounter;

    u16 invulnerableTimer;
    s16 shotTimer;
    u8 shotsFired;

    u16 score;

    Sprite *sprite;
    SpriteDefinition *spriteDef;

    Vect2D_f32 steering;
    u8 aiState;

    u16 steerTimer;

    u8 palette;
    u16 flags;

    u8 weapon;

    u8 active;    
} object_struct;

typedef struct object_struct ObjectStruct;

typedef ObjectStruct *(*ObjectCreateFunctionType)(fix16 x, fix16 y);
typedef void (*ObjectFunctionType)(ObjectStruct *objectStruct);
typedef void (*ObjectCollisionFunctionType)(ObjectStruct *objectStruct, ObjectStruct *collidedObject);

#endif