#include "Player.h"
#include "JoypadManager.h"
#include "LevelManager.h"
#include "Utils.h"
#include "Constants.h"
#include "Bullet.h"
#include "ObjectManager.h"

#include "sprite.h"

#include <genesis.h>

ObjectStruct playerStruct;
ObjectStruct *playerObject;

void PlayerUpdate(ObjectStruct *playerObject);
void PlayerDestroy(ObjectStruct *playerObject);
void PlayerDraw(ObjectStruct *playerObject);
void PlayerCollide(ObjectStruct *playerObject, ObjectStruct *collidedObject);

u16 bulletTimer = 0;

void PlayerSetup()
{
    playerObject = &playerStruct;
    playerObject->updateFunction = PlayerUpdate;
    playerObject->destroyFunction = PlayerDestroy;
    playerObject->drawFunction = PlayerDraw;
    playerObject->collisionFunction = PlayerCollide;
    playerObject->objectState = OBJECTSTATE_ACTIVE;

    playerObject->objectType = TYPE_PLAYER;

    playerObject->rectLeft = 0;
    playerObject->rectRight = 20;
    playerObject->rectTop = 0;
    playerObject->rectBottom = 32;


}

void PlayerInit()
{
    playerObject->isAlive = 1;
    playerObject->weapon = 0;

    playerObject->x = FIX16(100);
    playerObject->y = FIX16(100);

    playerObject->speedX = 0;
    playerObject->speedY = 0;
    playerObject->maxSpeedX = FIX16(2);
    playerObject->maxSpeedY = FIX16(2);

    playerObject->healthPoints = 10;

    playerObject->animationFrame = 2;
    playerObject->animationSubFrameCounter = 0;
    playerObject->invulnerableTimer = 0;
    playerObject->isAlive = 1;
    playerObject->lifetime = 0;

    playerObject->sprite = SPR_addSprite(&ship, 0, 0, TILE_ATTR(PAL3, TRUE, FALSE, FALSE));
}

void PlayerUpdate(ObjectStruct *playerObject)
{
    playerObject->speedX = 0;
    playerObject->speedY = 0;

    if (joystate & BUTTON_LEFT)
    {
        playerObject->speedX = -playerObject->maxSpeedX;
    }
    else if (joystate & BUTTON_RIGHT)
    {
        playerObject->speedX = playerObject->maxSpeedX;
    }

    if (joystate & BUTTON_UP)
    {
        playerObject->speedY = -playerObject->maxSpeedY;
    }
    else if (joystate & BUTTON_DOWN)
    {
        playerObject->speedY = playerObject->maxSpeedY;
    }

    if (joystickState.rightReleased || joystickState.leftReleased)
    {
        playerObject->animationSubFrameCounter = 0;
    }

    playerObject->x += playerObject->speedX;
    playerObject->y += playerObject->speedY;

    playerObject->x = min(playerObject->x, FIX16(VIRTUAL_SCREEN_WIDTH) - FIX16(16));
    playerObject->x = max(playerObject->x, 0);
    playerObject->y = min(playerObject->y, FIX16(VIRTUAL_SCREEN_HEIGHT) - FIX16(24));
    playerObject->y = max(playerObject->y, 0);

    if (bulletTimer == 0 && joystate & BUTTON_A)
    {
        ObjectStruct *bullet;
        fix16 speedY = FIX16(-7);
        fix16 speedX = fix16Mul(speedY, FIX16(0.7));
        if (playerObject->weapon == 0)
        {
            bulletTimer = 7;
            bullet = PlayerBulletCreate(playerObject->x + FIX16(11), playerObject->y - FIX16(8));
            if (bullet)
            {
                bullet->palette = PAL0;
                bullet->speedX = 0;
                bullet->speedY = speedY;
            }
        }
        else if (playerObject->weapon == 1)
        {
            bulletTimer = 10;
            for (int i = 0; i < 2; i++)
            {
                bullet = PlayerBulletCreate(playerObject->x + FIX16(7 + (i * 4 << 1)), playerObject->y - FIX16(8));
                if (bullet)
                {
                    bullet->palette = PAL0;
                    bullet->speedX = 0;
                    bullet->speedY = speedY;
                }
            }
        }
        else if (playerObject->weapon == 2)
        {
            bulletTimer = 10;
            for (int i = 0; i < 2; i++)
            {
                bullet = PlayerBulletCreate(playerObject->x + FIX16(7 + (i * 4 << 1)), playerObject->y - FIX16(8));
                if (bullet)
                {
                    bullet->palette = PAL0;
                    bullet->speedX = 0;
                    bullet->speedY = speedY;
                }
            }
            bullet = PlayerBulletCreate(playerObject->x + FIX16(7), playerObject->y - FIX16(8));
            if (bullet)
            {
                bullet->palette = PAL0;
                bullet->speedX = speedX;
                bullet->speedY = speedY;
            }
            bullet = PlayerBulletCreate(playerObject->x + FIX16(11), playerObject->y - FIX16(8));
            if (bullet)
            {
                bullet->palette = PAL0;
                bullet->speedX = -speedX;
                bullet->speedY = speedY;
            }
        }
    }

    if (playerObject->speedX < 0)
    {
        if (playerObject->animationSubFrameCounter > 7)
        {
            playerObject->animationSubFrameCounter = 0;
            playerObject->animationFrame = max(playerObject->animationFrame--, 0);
        }
    }
    else if (playerObject->speedX > 0)
    {
        if (playerObject->animationSubFrameCounter > 7)
        {
            playerObject->animationSubFrameCounter = 0;
            playerObject->animationFrame = min(playerObject->animationFrame++, 4);
        }
    }
    else
    {
        if (playerObject->animationFrame > 2)
        {
            if (playerObject->animationSubFrameCounter > 3)
            {
                playerObject->animationSubFrameCounter = 0;
                playerObject->animationFrame--;
            }
        }
        if (playerObject->animationFrame < 2)
        {
            if (playerObject->animationSubFrameCounter > 3)
            {
                playerObject->animationSubFrameCounter = 0;
                playerObject->animationFrame++;
            }
        }
    }

    SetObjectPosition(playerObject);
    playerObject->animationSubFrameCounter++;
    if (bulletTimer > 0)
        bulletTimer--;

    if (playerObject->invulnerableTimer > 0)
    {
        playerObject->invulnerableTimer--;
        if (playerObject->invulnerableTimer == 0)
        {
            SPR_setVisibility(playerObject->sprite, VISIBLE);
        }
        else
        {
            if (playerObject->invulnerableTimer % 5 == 0)
            {
                SPR_setVisibility(playerObject->sprite, VISIBLE);
            }
            else
            {
                SPR_setVisibility(playerObject->sprite, HIDDEN);
            }
        }
    }
}

void PlayerDraw(ObjectStruct *playerObject)
{
    SPR_setAnim(playerObject->sprite, playerObject->lifetime % 5 == 0);
    SPR_setFrame(playerObject->sprite, playerObject->animationFrame);
}

void IncreaseFirePower()
{
    if (playerObject->weapon < 2)
    {
        playerObject->weapon++;
    }
}

void PlayerCollide(ObjectStruct *playerObject, ObjectStruct *collidedObject)
{
    if (playerObject->invulnerableTimer > 0)
    {
        return;
    }

    if (((collidedObject->objectType == TYPE_ENEMY) ||
         (collidedObject->objectType == TYPE_ENEMY_PROJECTILE)))
    {
        if (playerObject->healthPoints > 1)
        {
            UpdateHP(playerObject->healthPoints);

            while (playerObject->weapon > 0)
            {
                playerObject->weapon--;
                PowerUpCreate(playerObject->x, playerObject->y);
            }

            playerObject->healthPoints--;
            playerObject->invulnerableTimer = 30;
        }
        else
        {
            playerObject->isAlive = 0;
            StartGame();
        }
    }
}

void PlayerDestroy(ObjectStruct *playerObject)
{
}
