#include "PlayerBullet.h"
#include "sprite.h"
#include "Utils.h"
#include "Player.h"
#include "ObjectManager.h"

void PlayerBulletUpdate(ObjectStruct *);
void PlayerBulletDestroy(ObjectStruct *);
void PlayerBulletDraw(ObjectStruct *);
void PlayerBulletCollide(ObjectStruct *, ObjectStruct *);

ObjectStruct *PlayerBulletCreate(fix16 x, fix16 y)
{
    ObjectStruct *object = ObjectManager_CreateObject(TYPE_PLAYER_PROJECTILE,
                                                      PlayerBulletUpdate,
                                                      PlayerBulletDestroy,
                                                      PlayerBulletDraw,
                                                      PlayerBulletCollide);

    if (!object)
    {
        KLog("Bullet could not be created! 1");
        return NULL;
    }

    object->x = x;
    object->y = y;
    object->speedX = 0;
    object->speedY = 0;
    object->accelX = FIX16(0.1);
    object->accelY = FIX16(0.1);
    object->animationFrame = 0;
    object->animationSubFrameCounter = 0;
    object->healthPoints = 1; 
    object->damage = 1;
    object->spriteDef = &playerbullet;

    if (!AddObjectPosition(object))
    {
        KLog("BulletCreate could not be created! 1");
        return NULL;
    }

    return object;
}

void PlayerBulletUpdate(ObjectStruct *object)
{
    if (object->objectState == OBJECTSTATE_INACTIVE)
    {
        return;
    }

    object->x += object->speedX;
    object->y += object->speedY;

    SetObjectPosition(object);
}

void PlayerBulletDestroy(ObjectStruct *object)
{

}

void PlayerBulletDraw(ObjectStruct *object)
{
}

void PlayerBulletCollide(ObjectStruct *object, ObjectStruct *collidedObject)
{
    object->objectState = OBJECTSTATE_INACTIVE;
}
