#include "PowerUp.h"
#include "sprite.h"
#include "Utils.h"

void PowerUpUpdate(ObjectStruct *);
void PowerUpDestroy(ObjectStruct *);
void PowerUpDraw(ObjectStruct *);
void PowerUpCollide(ObjectStruct *, ObjectStruct *);

ObjectStruct *PowerUpCreate(fix16 x, fix16 y)
{
    ObjectStruct *object = ObjectManager_CreateObject(TYPE_PICKUP,
                                                      PowerUpUpdate,
                                                      PowerUpDestroy,
                                                      PowerUpDraw,
                                                      PowerUpCollide);

    if (!object)
    {
        KLog("PowerUpCreate could not be created! 1");
        return NULL;
    }

    object->x = x;
    object->y = y;
    object->speedX = random() % 2 ? intToFix16(1) : intToFix16(-1);
    object->speedY = random() % 2 ? intToFix16(1) : intToFix16(-1);

    object->animationFrame = 0;
    object->animationSubFrameCounter = 0;
    object->palette = PAL3;
    object->spriteDef = &powerup;

    if (!AddObjectPosition(object))
    {
        KLog("PowerUpCreate could not be created! 1");
        return NULL;
    }

    return object;
}

void PowerUpUpdate(ObjectStruct *object)
{
    if (object->objectState == OBJECTSTATE_INACTIVE)
    {
        return;
    }

    object->x += object->speedX;
    object->y += object->speedY;
    if (object->y < 0)
    {
        object->y = 0;
        object->speedY = -object->speedY;
    }
    else if (object->y > intToFix16(VIRTUAL_SCREEN_HEIGHT - 16))
    {
        object->y = intToFix16(VIRTUAL_SCREEN_HEIGHT - 16);
        object->speedY = -object->speedY;
    }

    if (object->x < 0)
    {
        object->x = 0;
        object->speedX = -object->speedX;
    }
    else if (object->x > intToFix16(VIRTUAL_SCREEN_WIDTH - 16))
    {
        object->x = intToFix16(VIRTUAL_SCREEN_WIDTH - 16);
        object->speedX = -object->speedX;
    }

    if (object->lifetime > 1200)
    {
        object->objectState = OBJECTSTATE_INACTIVE;
        return;
    }
    else
    {
        if (object->lifetime > 1000)
        {
            if (object->lifetime % 4 == 0)
            {
                SPR_setVisibility(object->sprite, VISIBLE);
            }
            else
            {
                SPR_setVisibility(object->sprite, HIDDEN);
            }
        }
    }

    object->animationSubFrameCounter++;
    if (object->animationSubFrameCounter > 21)
    {
        object->animationFrame++;
        if (object->animationFrame > 1)
        {
            object->animationFrame = 0;
        }
        SPR_setFrame(object->sprite, object->animationFrame);
        object->animationSubFrameCounter = 0;
    }

    SetObjectPosition(object);
}

void PowerUpDestroy(ObjectStruct *object)
{
    SPR_releaseSprite(object->sprite);
}

void PowerUpDraw(ObjectStruct *object)
{

}

void PowerUpCollide(ObjectStruct *object, ObjectStruct *collidedObject)
{
    if (collidedObject->objectType != TYPE_PLAYER)
    {
        return;
    }

    if (collidedObject->invulnerableTimer > 0)
        return;

    IncreaseFirePower();

    object->objectState = OBJECTSTATE_INACTIVE;
}
