#include "Utils.h"
#include "LevelManager.h"
#include "Constants.h"
#include "sprite.h"
#include "Player.h"

char buffer[256];

void CollisionFunctionStub(ObjectStruct *objectStruct, ObjectStruct *collidedObject)
{
}

void SetObjectPosition(ObjectStruct *object)
{
    SPR_setPosition(object->sprite, HUD_WIDTH + fix16ToInt(object->x), fix16ToInt(object->y));
}

Sprite *AddObjectPosition(ObjectStruct *object)
{
    if (object->objectType != TYPE_PLAYER_PROJECTILE && object->objectType != TYPE_ENEMY_PROJECTILE)
    {
        object->sprite = SPR_addSprite(object->spriteDef, 0, 0, TILE_ATTR(object->palette, TRUE, FALSE, FALSE));
        if (object->sprite == NULL)
        {
            return NULL;
        }
    }

    s16 height = object->sprite->frame->h;
    s16 halfWidth = object->sprite->frame->h >> 1;
    if (object->y == 0)
    {
        object->y = object->y - intToFix16(height);
    }

    object->x = object->x - intToFix16(halfWidth);
    SetObjectPosition(object);

    return object->sprite;
}

void CommonEnemyUpdate(ObjectStruct *object)
{
    fix16 targetSpeedX = 0;
    fix16 targetSpeedY = 0;
    if (object->flags & VERTICAL_SIDE_DOWN)
        targetSpeedY = object->speed;
    if (object->flags & VERTICAL_SIDE_UP)
        targetSpeedY = -object->speed;

    if (object->flags & PATH_FLEE)
    {
        s16 fleeHeight = FLEE_HEIGHT;
        if (object->speedY < 0)
        {
            fleeHeight = SCREEN_HEIGHT - FLEE_HEIGHT;
        }
        if (object->y > intToFix16(fleeHeight))
        {
            if (object->x < intToFix16(VIRTUAL_SCREEN_WIDTH) - object->x)
                targetSpeedX = -object->speed;
            else
                targetSpeedX = object->speed;
        }
    }

    if (object->speedX < targetSpeedX)
        object->speedX += object->accelX;
    if (object->speedX > targetSpeedX)
        object->speedX -= object->accelX;

    if (object->speedY < targetSpeedY)
        object->speedY += object->accelY;
    if (object->speedY > targetSpeedY)
        object->speedY -= object->accelY;

    SPR_setVFlip(object->sprite, object->speedY < 0);

    object->x += object->speedX;
    object->y += object->speedY;

    SetObjectPosition(object);

    if (object->invulnerableTimer > 0)
    {
        object->invulnerableTimer--;
        if (object->invulnerableTimer == 0)
        {
            SPR_setPalette(object->sprite, object->palette);
        }
    }

    object->animationSubFrameCounter++;

    if (object->animationSubFrameCounter > 10)
    {
        object->animationFrame++;
        if (object->animationFrame > 1)
        {
            object->animationFrame = 0;
        }
        object->animationSubFrameCounter = 0;
    }
}