﻿using MetadataExtractor;
using MetadataExtractor.Formats.Exif;
using MetadataExtractor.Formats.Png;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace consolenet
{
    class Program
    {
        static List<Color> genesisColors = new List<Color>();
        static Dictionary<Color, Color> dicColor = new Dictionary<Color, Color>();



        static void Main(string[] args)
        {
            //CropFiles(@"E:\spaceblastingMD\git\res\gfx\title\background");
            //convertSize(@"E:\spaceblastingMD\git\res\gfx\title\lol", 16);
            //GenerateAngles();
            //FindBestColors(@"E:\spaceblastingMD\git\res\sprite\sprites");
        }

        private static void GenerateAngles()
        {
            float steps = 90;
            for (float i = 0; i <= 180; i += steps)
            {
                double r = i;
                Debug.WriteLine("FIX16({0}), FIX16({1}),", Math.Cos(r), Math.Sin(r));
            }
        }

        private static void CropFiles(string directory)
        {


            foreach (var file in System.IO.Directory.GetFiles(directory))
            {
                if (!file.EndsWith(".png"))
                    continue;
                var directories = ImageMetadataReader.ReadMetadata(file);

                var subIfdDirectory = directories.OfType<PngDirectory>().FirstOrDefault();
                var width = subIfdDirectory?.GetInt32(PngDirectory.TagImageWidth);
                var height = subIfdDirectory?.GetInt32(PngDirectory.TagImageHeight);

                var sectionsX = Math.Ceiling(width.Value / 128f);
                var sectionsY = Math.Ceiling(height.Value / 128f);
                if (sectionsX > 1 || sectionsY > 1)
                {
                    for (int i = 0; i < sectionsX; i++)
                    {
                        for (int j = 0; j < sectionsY; j++)
                        {
                            CropImage(file, i * 128, j * 128, Math.Min(128, width.Value - (i * 128)), Math.Min(128, height.Value - (j * 128)), i + 1, j + 1);
                        }
                    }
                }
            }
        }

        private static void CropImage(string file, int offsetX, int offsetY, int cropWidth, int cropHeight, int i, int j)
        {
            string line = string.Format("magick convert \"{0}\" -crop {1}x{2}+{3}+{4} \"{5}\"", file, cropWidth, cropHeight, offsetX, offsetY, Path.GetDirectoryName(file) + "\\" + i + "x" + j + Path.GetFileName(file));
            Process process = new Process();
            ProcessStartInfo startInfo = new ProcessStartInfo();
            startInfo.WindowStyle = ProcessWindowStyle.Hidden;
            startInfo.FileName = "cmd.exe";
            startInfo.Arguments = "/C " + line;
            process.StartInfo = startInfo;
            process.Start();
            Debug.WriteLine("\"{0}\" {1}x{2}+{3}+{4}", i + "x" + j + Path.GetFileName(file), cropWidth, cropHeight, offsetX, offsetY);
        }

        private static void FindBestColors(string directory)
        {
            for (int i = 0; i < 255; i += 34)
            {
                for (int j = 0; j < 255; j += 34)
                {
                    for (int k = 0; k < 255; k += 34)
                    {
                        genesisColors.Add(Color.FromArgb(i, j, k));
                    }
                }
            }

            foreach (var file in System.IO.Directory.GetFiles(directory))
            {
                if (!file.EndsWith(".png"))
                    continue;
                var bmp = (Bitmap)Image.FromFile(file);
                ReplaceColor(bmp);
                var encoderParameters = new EncoderParameters(1);
                encoderParameters.Param[0] = new EncoderParameter(System.Drawing.Imaging.Encoder.Quality, 1L);
                bmp.Save(Path.GetDirectoryName(file) + "\\best\\" + Path.GetFileName(file), GetEncoder(ImageFormat.Png), encoderParameters);
            }
        }

        private static ImageCodecInfo GetEncoder(ImageFormat format)
        {
            var codecs = ImageCodecInfo.GetImageDecoders();
            foreach (var codec in codecs)
            {
                if (codec.FormatID == format.Guid)
                {
                    return codec;
                }
            }
            return null;
        }

        private static double ColourDistance(Color e1, Color e2)
        {
            long rmean = ((long)e1.R + (long)e2.R) / 2;
            long r = (long)e1.R - (long)e2.R;
            long g = (long)e1.G - (long)e2.G;
            long b = (long)e1.B - (long)e2.B;
            return Math.Sqrt((((512 + rmean) * r * r) >> 8) + 4 * g * g + (((767 - rmean) * b * b) >> 8));
        }

        private static void ReplaceColor(Bitmap bmp)
        {
            var lockedBitmap = new LockBitmap(bmp);
            lockedBitmap.LockBits();

            for (int y = 0; y < lockedBitmap.Height; y++)
            {
                for (int x = 0; x < lockedBitmap.Width; x++)
                {
                    var oldColor = lockedBitmap.GetPixel(x, y);
                    if (dicColor.ContainsKey(oldColor))
                    {
                        lockedBitmap.SetPixel(x, y, dicColor[oldColor]);
                    }
                    else
                    {
                        Dictionary<Color, double> imageDictionary = new Dictionary<Color, double>();
                        foreach (var color in genesisColors)
                        {
                            var dist = ColourDistance(oldColor, color);
                            imageDictionary[color] = dist;
                        }
                        imageDictionary = imageDictionary.OrderBy(d => d.Value).ToDictionary(d => d.Key, d => d.Value);
                        Color newColor = imageDictionary.FirstOrDefault().Key;
                        int i = 1;
                        while (dicColor.ContainsValue(newColor) && i <= dicColor.Count)
                        {
                            newColor = imageDictionary.Skip(i++).FirstOrDefault().Key;
                        }
                        lockedBitmap.SetPixel(x, y, newColor);
                        dicColor[oldColor] = newColor;
                    }
                }
            }
            lockedBitmap.UnlockBits();
        }

        private static void convertSize(string directory, int tileSize)
        {
            foreach (var file in System.IO.Directory.GetFiles(directory))
            {
                if (!file.EndsWith(".png"))
                    continue;
                var directories = ImageMetadataReader.ReadMetadata(file);

                var subIfdDirectory = directories.OfType<PngDirectory>().FirstOrDefault();
                var width = subIfdDirectory?.GetInt32(PngDirectory.TagImageWidth);
                var height = subIfdDirectory?.GetInt32(PngDirectory.TagImageHeight);
                if (width / tileSize != 0)
                {
                    width = tileSize * ((width - 1) / tileSize + 1);
                }
                if (height / tileSize != 0)
                {
                    height = tileSize * ((height - 1) / tileSize + 1);
                }
                string line = string.Format("magick convert \"{3}\" -background none -gravity SouthEast -extent {0}x{1} \"{2}\"", width, height, file, file);
                Process process = new Process();
                ProcessStartInfo startInfo = new ProcessStartInfo();
                startInfo.WindowStyle = ProcessWindowStyle.Hidden;
                startInfo.FileName = "cmd.exe";
                startInfo.Arguments = "/C " + line;
                process.StartInfo = startInfo;
                process.Start();
                Debug.WriteLine(line);
                Debug.WriteLine("//" + Path.GetFileName(file) + " " + width / tileSize + " " + height / tileSize + " " + width + " " + height);
            }
        }

        private static void generatecollision(string[] args)
        {
            string[] lines = File.ReadAllLines(args[0]);
            string text = string.Empty;
            int count = 0;
            foreach (var line in lines)
            {
                var tiles = line.Split(',');
                for (int i = 0; i < tiles.Count(); i++)
                {
                    if (tiles[i] != "0")
                    {
                        tiles[i] = "1";
                    }
                }
                count += tiles.Count();
                text += string.Join(",", tiles);
                text += ",\r\n";
            }
            text += "};";
            File.WriteAllText("level001.h", "#ifndef LEVEL001_INCLUDE_H\r\n#define LEVEL001_INCLUDE_H\r\n#include <genesis.h>\r\nextern u8 const level001[" + count + "];\r\n#endif");
            string header = "#include \"level001.h\" \r\nu8 const level001[" + count + "]={\r\n";
            File.WriteAllText("level001.c", header + text);
        }
    }
}
